
/** Projet ETI3 - CPE Lyon - 2010-2011 */



#include <glut_helper.hpp>


#include <GL/glut.h>
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <vector>


#include <tinyxml.h>


//draw the model
void draw_scene();
//load the model
void load_model();
//compute a random number
double get_rand();



//glut functions
int setup_callback();
static void display_callback();
static void reshape_callback (int width, int height);
static void keyboard_callback(unsigned char key,int x,int y);
static void idle_callback(int value);
static void motion_callback (int x, int y);
static void mouse_callback (int button, int action, int x, int y);



using namespace cpe;




//GLOBAL VARIABLES
//************************************//
unsigned int time_anim=0;
unsigned int counter=0;
navigator_tool nav;



std::vector<double> cylinder_vertex;
std::vector<double> cylinder_normal;
std::vector<int> cylinder_face;


//load the model
void load_model()
{
    //preprocessing
    std::cout<<"<DO PREPROCESSING ...>"<<std::endl;


    {
        std::string filename="data/circuit1.xml";
        std::cout<<"Read xml file "<<filename<<" ..."<<std::endl;

        TiXmlDocument doc("data/circuit1.xml");doc.LoadFile();
        TiXmlElement *root=doc.FirstChildElement("circuit");
        TiXmlNode *node=root->FirstChild();
        int counter=0;
        do
        {
            if(std::string(node->Value())=="edge")
                ++counter;
            node=node->NextSibling();
        }while(node!=NULL);

        std::cout<<"File [OK] found "<<counter<<" edges "<<std::endl;

    }

    //... do all my pre-processing stuff



    //********************************************//
    //precompute cylinder for instance
    //********************************************//

    unsigned int N=30;

    double r=0.1;
    double L=0.8;
    for(unsigned int k=0;k<N;++k)
    {
        double alpha=k/double(N);

        //vertex
        cylinder_vertex.push_back(r*cos(2*M_PI*alpha));
        cylinder_vertex.push_back(r*sin(2*M_PI*alpha)-0.4);
        cylinder_vertex.push_back(0);

        cylinder_vertex.push_back(r*cos(2*M_PI*alpha));
        cylinder_vertex.push_back(r*sin(2*M_PI*alpha)-0.4);
        cylinder_vertex.push_back(L);


        //normal
        cylinder_normal.push_back(-sin(2*M_PI*alpha));
        cylinder_normal.push_back( cos(2*M_PI*alpha));
        cylinder_normal.push_back(0.0);

        cylinder_normal.push_back(-sin(2*M_PI*alpha));
        cylinder_normal.push_back( cos(2*M_PI*alpha));
        cylinder_normal.push_back(0);
    }

    //face
    for(unsigned int k=0;k<N;++k)
    {
        cylinder_face.push_back(2*k);
        cylinder_face.push_back(2*k+1);
        cylinder_face.push_back(2*((k+1)%N)+1);
        cylinder_face.push_back(2*((k+1)%N)+0);
    }



    std::cout<<"</DO PREPROCESSING> "<<std::endl<<std::endl;
}


//Draw the scene
void draw_scene()
{
    ++time_anim;

    glEnable(GL_LIGHTING);

    //... do all my drawing




    //some examples :

    //*************************************//
    //draw triangle
    //*************************************//

    glBegin(GL_TRIANGLES);
    glNormal3d(0,0,1);
    glColor3d(1,0,0);
    glVertex3d(0,0.4,0);

    glColor3d(0,0,1);
    glVertex3d(0.4,0.4,0.3);

    glColor3d(0,1,0);
    glVertex3d(0,0.6,0);
    glEnd();

    //*************************************//
    //draw cube
    //*************************************//

    glBegin(GL_QUADS);

    double a=0.3;

    glNormal3d(0,0,1); glVertex3d(0,0,0); glVertex3d(0,a,0);  glVertex3d(a,a,0); glVertex3d(a,0,0);
    glNormal3d(0,0,-1); glVertex3d(0,0,a); glVertex3d(a,0,a);  glVertex3d(a,a,a); glVertex3d(0,a,a);
    glNormal3d(0,1,0); glVertex3d(0,0,0); glVertex3d(a,0,0);  glVertex3d(a,0,a); glVertex3d(0,0,a);
    glNormal3d(0,-1,0); glVertex3d(0,a,0); glVertex3d(0,a,a);  glVertex3d(a,a,a); glVertex3d(a,a,0);
    glNormal3d(1,0,0); glVertex3d(0,0,0); glVertex3d(0,0,a);  glVertex3d(0,a,a); glVertex3d(0,a,0);
    glNormal3d(-1,0,0); glVertex3d(a,0,0); glVertex3d(a,a,0);  glVertex3d(a,a,a); glVertex3d(a,0,a);
    glEnd();


    //*************************************//
    //draw cylinder
    //*************************************//


    //store vertex
    glBegin(GL_QUADS);
    for(unsigned int k=0;k<cylinder_face.size()/4;++k)
    {
        for(unsigned int k2=0;k2<4;++k2)
        {
            double *p_normal=&cylinder_normal[0]+3*cylinder_face[4*k+k2];
            glNormal3d(p_normal[0],p_normal[1],p_normal[2]);

            double *p_vertex=&cylinder_vertex[0]+3*cylinder_face[4*k+k2];

            glColor3d(1-p_vertex[2],1-p_vertex[2],p_vertex[2]);
            glVertex3d(p_vertex[0],p_vertex[1],p_vertex[2]);
        }
    }
    glEnd();

}















int main(int argc, char *argv[])
{
    std::cout<<"**hello**\n"<<std::endl;
    std::cout<<"run "<<argv[0]<<" with "<<argc-1<<" parameters ... \n"<<std::endl;


  // ************* //
  // start OpenGL
  // ************* //

  //window size
  nav.screen_size_x()=800;
  nav.screen_size_y()=800;



  //init opengl & glut
  setup_opengl(argc,argv,nav);
  //setup callback functions
  setup_callback();





  // ************* //
  // RUN
  // ************* //
  print_help_start();

  //do start once
  load_model();

  //glut loop
  std::cout<<"<RUN MAIN LOOP ...>"<<std::endl;
  glutMainLoop ();


  return 0;
}



// *******************//
// Helper
// *******************//

//just a rand number \in [0,1]
double get_rand()
{
  int precision=50000;
  return double(rand()%precision)/double(precision);
}






//*******************//
//GLUT STUFF
//*******************//


//glut functions
int setup_callback()
{

    //callback
    glutDisplayFunc (display_callback);
    glutReshapeFunc (reshape_callback);
    glutKeyboardFunc(keyboard_callback);
    glutTimerFunc(33,idle_callback,1);
    glutMotionFunc (motion_callback);
    glutMouseFunc (mouse_callback);

    return 0;
}



//display
static void display_callback()
{
    display_manager_begin(nav);
    draw_scene();
    display_manager_end(nav,counter);
}


static void reshape_callback (int width, int height) {reshape_manager(width,height,nav);}
static void idle_callback(int value) {value=1;glutPostRedisplay ();glutTimerFunc( 20, idle_callback, 1);}
static void keyboard_callback(unsigned char key,int x,int y) {keyboard_manager(key,x,y,nav);}
static void motion_callback (int x, int y) {motion_manager(x,y,nav);}
static void mouse_callback (int button, int action, int x, int y) {mouse_manager(button,action,x,y,nav);}












